'use strict';

var paginator = angular.module('crispy-paginator', []);

paginator.provider('CrispyPaginator', [
  function () {
    var self = this;

    this.templateUrl = 'crispy-paginator.tpl.html';
    this.visiblePages = 10;
    this.useBoundaryLinks = true;
    this.alwaysShowBoundaryPages = false;

    this.$get = function () {
      return {
        getSetting: function (name) {
          return self[name];
        }
      };
    };

    this.setTemplateUrl = function (url) {
      this.templateUrl = url;
    };

    this.setVisiblePages = function (number) {
      this.visiblePages = number;
    };

    this.setUseBoundaryLinks = function (boundaryLinks) {
      this.useBoundaryLinks = !!boundaryLinks;
    };

    this.setAlwaysShowBoundaryPages = function (showBoundaryPages) {
      this.alwaysShowBoundaryPages = !!showBoundaryPages;
    };
  }
]);

paginator.controller('PaginatorCtrl', ['$scope', '$location', function ($scope, $location) {
  var self = this;
  var previousPage;

  /**
   * Selects given `pageNumber`.
   */
  $scope.selectPage = function(pageNumber) {
    $scope.currentPage = pageNumber;
  };

  /**
   * Watches changes to `currentPage` and renders the paginator accordingly. If
   * user provides incorrect page number, first page is rendered.
   */
  $scope.$watch('currentPage', function () {
    if (!self.isCorrectPage($scope.currentPage)) {
      $scope.selectPage(1);
    }

    self.render();
  });

  /**
   * Watches changes to `totalPages` and renders the pagintor accordingly.
   */
  $scope.$watch('totalPages', function() {
    self.render();
  });

  if ($scope.useLocation === true) {
    /**
     * Watches for changes in location and changes `currentPage` accordingly.
     * This is useful if `$location` service is set to HTML5 mode.
     */
    $scope.$watch(function() {
      return _.parseInt($location.search().page) || 1;
    }, function(page) {
      $scope.currentPage = page;
    });
  }

  /**
   * Checks if `page` is integer within boundaries of 1 and `$scope.totalPages`.
   * If `$scope.totalPages` isn't set or is null, we'll temporarily assume
   * everything is alright.
   */
  this.isCorrectPage = function (page) {
    if ($scope.totalPages === null || $scope.totalPages === undefined) {
      return true;
    }

    page = _.parseInt(page, 10);
    var isPageInBoundaries = page <= $scope.totalPages && page >= 1;

    if (!isPageInBoundaries || isNaN(page)) {
      return false;
    }

    return true;
  };

  /**
   * Renders paginator and changes querystring in case `$scope.totalPages` is
   * set and not null.
   */
  this.render = function () {
    if ($scope.totalPages !== null && $scope.totalPages !== undefined) {
      $scope.pages = self.createPages($scope.totalPages);

      if ($scope.useLocation === true) {
        /**
         * Don't change querystring if `changePage1Location` is false and current
         * and previous pages are 1 or undefined.
         */
        if ((previousPage === undefined || previousPage === 1) &&
            $scope.currentPage === 1 && $scope.changePage1Location !== true) {
          previousPage = $scope.currentPage;

          return;
        }

        previousPage = $scope.currentPage;
        $location.search('page', $scope.currentPage);
      }
    }
  };
}]);

paginator.directive('crispyPaginator', function (CrispyPaginator) {
  return {
    restrict: 'E',
    templateUrl: function(element, attr) {
      if (attr.templateUrl) {
        return attr.templateUrl;
      }

      return CrispyPaginator.getSetting('templateUrl');
    },
    controller: 'PaginatorCtrl',
    require: 'crispyPaginator',
    scope: {
      'totalPages': '=',
      'currentPage': '=',
      'useLocation': '=',
      'changePage1Location': '=',
    },
    link: function (scope, element, attrs, ctrl) {
      var optionalSettings = {
        'visiblePages': 'int',
        'useBoundaryLinks': 'bool',
        'alwaysShowBoundaryPages': 'bool',
      };

      /**
       * Helper for getting setting either from `attrs` or from provider.
       * Settings are always cast to appropriate type depending on
       * `optionalSettings`.
       */
      ctrl.getSetting = function (name) {
        if (angular.isDefined(attrs[name])) {
          var value = scope.$eval(attrs[name]);

          switch (optionalSettings[name]) {
            case 'int':
              value = _.parseInt(value);
              break;
            case 'bool':
              value = !!value;
              break;
          }

          return value;
        } else {
          return CrispyPaginator.getSetting(name);
        }
      };

      var visiblePages = ctrl.getSetting('visiblePages');
      var useBoundaryLinks = ctrl.getSetting('useBoundaryLinks');
      var alwaysShowBoundaryPages = ctrl.getSetting('alwaysShowBoundaryPages');

      ctrl.createPages = function (totalPages) {
        // jshint maxcomplexity:12
        var pages = [];
        var currentPage = _.parseInt(scope.currentPage, 10);
        var firstPage = 1;
        var lastPage = totalPages;

        // If there are more total pages than allowed visible pages, we'll cut
        // pages to given limit.
        if (lastPage > visiblePages) {
          firstPage = Math.max(currentPage - Math.floor(visiblePages / 2), 1);
          lastPage = firstPage + visiblePages - 1;

          if (lastPage > totalPages) {
            lastPage = totalPages;
            firstPage = lastPage - visiblePages + 1;
          }
        }

        _.each(_.range(firstPage, lastPage + 1), function (page) {
          pages.push({
            'text': page,
            'active': page == currentPage,
            'disabled': false,
            'number': page,
          });
        });

        if (alwaysShowBoundaryPages) {
          if (firstPage > 2) {
            pages.unshift({
              'text': '...',
              'active': false,
              'disabled': false,
              'number': null,
            });
          }

          if (firstPage > 1) {
            pages.unshift({
              'text': 1,
              'active': false,
              'disabled': false,
              'number': 1,
            });
          }

          if (lastPage < totalPages - 1) {
            pages.push({
              'text': '...',
              'active': false,
              'disabled': false,
              'number': null,
            });
          }

          if (lastPage < totalPages) {
            pages.push({
              'text': totalPages,
              'active': false,
              'disabled': false,
              'number': totalPages,
            });
          }
        }

        pages.unshift({
          'text': 'Previous',
          'active': false,
          'disabled': currentPage == 1,
          'number': currentPage - 1 < firstPage ? firstPage : currentPage - 1,
        });

        pages.push({
          'text': 'Next',
          'active': false,
          'disabled': currentPage === parseInt(scope.totalPages),
          'number': currentPage + 1 > lastPage ? lastPage : currentPage + 1,
        });

        if (useBoundaryLinks) {
          pages.unshift({
            'text': 'First',
            'active': false,
            'disabled': currentPage == 1,
            'number': 1,
          });

          pages.push({
            'text': 'Last',
            'active': false,
            'disabled': currentPage == totalPages,
            'number': totalPages,
          });
        }

        return pages;
      };
    },
  };
});
