'use strict';

describe('Paginator directive', function () {
  var $rootScope;
  var $compile;
  var element;

  beforeEach(module('crispy-paginator'));

  beforeEach(module('crispy-paginator.templates'));

  beforeEach(inject(function (_$compile_, _$rootScope_) {
    $rootScope = _$rootScope_.$new();
    $rootScope.page = 2;
    $rootScope.totalPages = 3;

    $compile = _$compile_;

    element = $compile('<crispy-paginator total-pages="totalPages" current-page="page">' +
      '</crispy-paginator>')($rootScope);
    $rootScope.$digest();
  }));

  it('should load the default template', function () {
    element = $compile('<crispy-paginator>')($rootScope);
    $rootScope.$digest();

    expect(element.html()).toContain('<div class="pagination">');
  });

  it('should load a custom template', inject(function ($httpBackend) {
    $httpBackend.when('GET', 'test-template.tpl.html').respond('test-template');

    element = $compile('<crispy-paginator template-url="test-template.tpl.html">' +
      '</crispy-paginator>')($rootScope);
    $rootScope.$digest();
    $httpBackend.flush();

    $httpBackend.expectGET('test-template.tpl.html');
    expect(element.html()).toContain('test-template');
  }));

  it('contains correct number of page elements', function () {
    expect(element.find('li').length).toBe(7);
  });

  it('sets current page as active', function () {
    expect(getPageElement(4).hasClass('active')).toBeTruthy();
  });

  it('does not set inactive pages as active', function () {
    _.each([1, 2, 3, 5, 6, 7], function (page) {
      expect(getPageElement(page).hasClass('active')).toBeFalsy();
    });
  });

  it('should change page on location change', inject(function ($location) {
    element = $compile('<crispy-paginator total-pages="3" current-page="page" ' +
      'use-location="true"></crispy-paginator>')($rootScope);
    $rootScope.$location = $location;
    $rootScope.$digest();

    $location.search({'page': 1});
    $rootScope.$digest();

    expect(getPageElement(3).hasClass('active')).toBeTruthy();
  }));

  it('should change to page 1 if user requests non-existant page', function () {
    element = $compile('<crispy-paginator total-pages="3" current-page="page" ' +
      '></crispy-paginator>')($rootScope);
    $rootScope.page = 4;
    $rootScope.$digest();

    expect(getPageElement(3).hasClass('active')).toBeTruthy();
  });

  it('should change to page 1 if user requests non-integer page', function () {
    element = $compile('<crispy-paginator total-pages="3" current-page="page">' +
      '</crispy-paginator>')($rootScope);
    $rootScope.page = 'a';
    $rootScope.$digest();

    expect(getPageElement(3).hasClass('active')).toBeTruthy();
  });

  it('should change to page 1 if `currentPage` is undefined or null', function () {
    element = $compile('<crispy-paginator total-pages="3" current-page="page">' +
      '</crispy-paginator>')($rootScope);
    $rootScope.page = null;
    $rootScope.$digest();

    expect(getPageElement(3).hasClass('active')).toBeTruthy();
  });

  it('should set the first page\'s "previous" page as the current one', function () {
    $rootScope.page = 1;
    $rootScope.$digest();

    click(getPageElement(2).find('a')[0]);

    expect(getPageElement(3).hasClass('active')).toBeTruthy();
  });

  it('should set the last page\'s "next" page as the current one', function () {
    $rootScope.page = 3;
    $rootScope.$digest();

    click(getPageElement(6).find('a')[0]);

    expect(getPageElement(5).hasClass('active')).toBeTruthy();
  });

  it('should show surrounding 10 pages by default', function () {
    element = $compile('<crispy-paginator total-pages="30" current-page="20">' +
      '</crispy-paginator>')($rootScope);
    $rootScope.page = null;
    $rootScope.$digest();

    expect(element.find('li').length).toBe(14);
  });

  it('should show surrounding pages set by user', function () {
    element = $compile('<crispy-paginator total-pages="120" current-page="1" ' +
      'visible-pages="20"></crispy-paginator>')($rootScope);
    $rootScope.page = null;
    $rootScope.$digest();

    expect(element.find('li').length).toBe(24);
  });

  describe('with useBoundaryLinks set', function () {
    it('by default should contain first and last links', function () {
      element = $compile('<crispy-paginator total-pages="totalPages" current-page="page">' +
        '</crispy-paginator>')($rootScope);
      $rootScope.$digest();

      expect(getLinks().length).toBe(7);
      expect(getLinks()[0].text).toBe('First');
      expect(getLinks()[getLinks().length - 1].text).toBe('Last');
    });

    it('to false should not contain first and last links', function () {
      element = $compile('<crispy-paginator total-pages="totalPages" current-page="page" ' +
        'use-boundary-links="false"></crispy-paginator>')($rootScope);
      $rootScope.$digest();

      expect(getLinks().length).toBe(5);
      expect(getLinks()[0].text).not.toBe('First');
      expect(getLinks()[getLinks().length - 1].text).not.toBe('Last');
    });

    it('to true should contain first and last links', function () {
      element = $compile('<crispy-paginator total-pages="totalPages" current-page="page" ' +
        'use-boundary-links="true"></crispy-paginator>')($rootScope);
      $rootScope.$digest();

      expect(getLinks().length).toBe(7);
      expect(getLinks()[0].text).toBe('First');
      expect(getLinks()[getLinks().length - 1].text).toBe('Last');
    });
  });

  describe('with alwaysShowBoundaryPages set', function () {
    it('by default should not show first and last page links', function () {
      element = $compile('<crispy-paginator total-pages="totalPages" current-page="page">' +
        '</crispy-paginator>')($rootScope);
      $rootScope.totalPages = 100;
      $rootScope.page = 50;
      $rootScope.$digest();

      var containsBoundaryPageLinks = _.any(getLinks(), function (value) {
        return value.text === '1' || value.text === '100';
      });

      expect(containsBoundaryPageLinks).toBeFalsy();
    });

    it('to true should show first and last page links', function () {
      element = $compile('<crispy-paginator total-pages="totalPages" current-page="page" ' +
        'always-show-boundary-pages="true"></crispy-paginator>')($rootScope);
      $rootScope.totalPages = 100;
      $rootScope.page = 50;
      $rootScope.$digest();

      expect(getLinks()[2].text).toBe('1');
      expect(getLinks()[getLinks().length - 3].text).toBe('100');
    });

    it('to false should not show first and last page links', function () {
      element = $compile('<crispy-paginator total-pages="totalPages" current-page="page" ' +
        'always-show-boundary-pages="false"></crispy-paginator>')($rootScope);
      $rootScope.totalPages = 100;
      $rootScope.page = 50;
      $rootScope.$digest();

      var containsBoundaryPageLinks = _.any(getLinks(), function (value) {
        return value.text === '1' || value.text === '100';
      });

      expect(containsBoundaryPageLinks).toBeFalsy();
    });

    it('to true should not show first page twice if it is already in visible range', function () {
      element = $compile('<crispy-paginator total-pages="totalPages" current-page="page" ' +
        'always-show-boundary-pages="true"></crispy-paginator>')($rootScope);
      $rootScope.totalPages = 100;
      $rootScope.page = 2;
      $rootScope.$digest();

      expect(getLinks()[2].text).toBe('1');
      expect(getLinks()[3].text).not.toBe('1');
    });

    it('to true should not show last page twice if it is already in visible range', function () {
      element = $compile('<crispy-paginator total-pages="totalPages" current-page="page" ' +
        'always-show-boundary-pages="true"></crispy-paginator>')($rootScope);
      $rootScope.totalPages = 100;
      $rootScope.page = 99;
      $rootScope.$digest();

      expect(getLinks()[getLinks().length - 3].text).toBe('100');
      expect(getLinks()[getLinks().length - 4].text).not.toBe('100');
    });
  });

  function getLinks() {
    return element.find('a');
  }

  function getPageElement(page) {
    return element.find('li').eq(page - 1);
  }

  /**
   * Helper that allows for proper PhantomJS clicks.
   *
   * Taken from http://stackoverflow.com/a/15948355/206720.
   */
  function click (el) {
    var ev = document.createEvent('MouseEvent');
    ev.initMouseEvent(
        'click',
        true /* bubble */, true /* cancelable */,
        window, null,
        0, 0, 0, 0, /* coordinates */
        false, false, false, false, /* modifier keys */
        0 /*left*/, null
    );
    el.dispatchEvent(ev);
  }
});
